/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/fs/fs.hpp>
#include <ox/model/desctypes.hpp>
#include <ox/std/buffer.hpp>
#include <ox/std/size.hpp>

#include <keel/context.hpp>
#include <turbine/context.hpp>

#include "initparams.hpp"

namespace nostalgia::core {
class Context;
}

namespace nostalgia::core::gl {
void drawMainView(core::Context*) noexcept;
void setRenderSize(core::Context*, int width, int height) noexcept;
ox::Size getRenderSize(core::Context*) noexcept;
void clearRenderSize(core::Context *ctx) noexcept;
}

namespace nostalgia::core {

// User Input Output
class Context {
	friend bool bgStatus(Context *ctx, unsigned bg) noexcept;
	friend ox::Error initGfx(Context *ctx, const InitParams&) noexcept;
	friend void shutdownGfx(Context *ctx) noexcept;
	friend ox::Error loadBgTileSheet(Context *ctx,
	                                 unsigned cbb,
	                                 const ox::FileAddress &tilesheetPath,
	                                 const ox::FileAddress &palettePath) noexcept;
	friend ox::Error loadSpriteTileSheet(Context *ctx,
	                                     const ox::FileAddress &tilesheetAddr,
	                                     const ox::FileAddress &paletteAddr) noexcept;
	friend ox::Result<struct TileSheetData> loadTileSheet(Context *ctx,
	                                                      const struct CompactTileSheet &tilesheetAddr) noexcept;
	friend uint8_t bgStatus(Context *ctx) noexcept;
	friend void clearTileLayer(Context *ctx, unsigned bgIdx) noexcept;
	friend void draw(Context *ctx) noexcept;
	friend void setBgCbb(Context *ctx, unsigned bgIdx, unsigned cbb) noexcept;
	friend void setBgStatus(Context *ctx, uint32_t status) noexcept;
	friend void setBgStatus(Context *ctx, unsigned bg, bool status) noexcept;
	friend void setTile(Context *ctx, unsigned bgIdx, int column, int row, uint8_t tile) noexcept;
	friend void setSprite(Context *ctx,
	                      unsigned idx,
	                      int x,
	                      int y,
	                      unsigned tileIdx,
	                      unsigned spriteShape,
	                      unsigned spriteSize,
	                      unsigned flipX) noexcept;
	friend void hideSprite(Context *ctx, unsigned idx) noexcept;
	friend void gl::drawMainView(core::Context*) noexcept;
	friend void gl::setRenderSize(core::Context*, int width, int height) noexcept;
	friend ox::Size gl::getRenderSize(core::Context*) noexcept;
	friend void gl::clearRenderSize(core::Context *ctx) noexcept;

	public:
		turbine::Context *turbineCtx = nullptr;

	public:
		Context() noexcept = default;

		Context(Context &other) noexcept = delete;
		Context(const Context &other) noexcept = delete;
		Context(const Context &&other) noexcept = delete;

		~Context() noexcept;

		auto &rom() noexcept {
			return *turbineCtx->rom;
		}

};

}

